/*
 * Simulator of MCS51 (brkcl.h)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#ifndef BRKCL_HEADER
#define BRKCL_HEADER

#include "ddconfig.h"

#include "pobj.h"
#include "stypes.h"


/*
 * Base object of breakpoints
 */

class brkcl: public TObject
{
public:
  uint  addr;
  enum  brk_perm perm;  // permanency (FIX,DYNAMIC)
  int   hit;
  int   cnt;

  brkcl(uint Iaddr,
	enum brk_perm  Iperm,
	int  Ihit);
  virtual ~brkcl(void);

  virtual enum brk_type type(void)= 0;
};


/*
 * FETCH type of breakpoints
 */

class brk_fetch: public brkcl
{
public:
  uchar code;

  brk_fetch(uint Iaddr,
	    enum brk_perm Iperm,
	    int  Ihit);

  virtual enum brk_type type(void);
};


/*
 * Base of EVENT type of breakpoints
 */

class brk_ev: public brkcl
{
public:
  brk_ev(uint Iaddr,
	 enum brk_perm Iperm,
	 int  Ihit,
	 enum brk_event Ievent,
	 const char *Iid);

  enum brk_event event;
  const char *id;

  virtual enum brk_type type(void);
  virtual bool match(struct event_rec *ev);
};


/* 
 * WRITE IRAM
 */

class brk_wi: public brk_ev
{
public:
  brk_wi(uint Iaddr,
	 enum brk_perm Iperm,
	 int  Ihit);

  virtual bool match(struct event_rec *ev);
};


/* 
 * READ IRAM
 */

class brk_ri: public brk_ev
{
public:
  brk_ri(uint Iaddr,
	 enum brk_perm Iperm,
	 int  Ihit);

  virtual bool match(struct event_rec *ev);
};


/* 
 * WRITE XRAM
 */

class brk_wx: public brk_ev
{
public:
  brk_wx(uint Iaddr,
	 enum brk_perm Iperm,
	 int  Ihit);

  virtual bool match(struct event_rec *ev);
};


/* 
 * READ XRAM
 */

class brk_rx: public brk_ev
{
public:
  brk_rx(uint Iaddr,
	 enum brk_perm Iperm,
	 int  Ihit);

  virtual bool match(struct event_rec *ev);
};


/* 
 * WRITE SFR
 */

class brk_ws: public brk_ev
{
public:
  brk_ws(uint Iaddr,
	 enum brk_perm Iperm,
	 int  Ihit);

  virtual bool match(struct event_rec *ev);
};


/* 
 * READ SFR
 */

class brk_rs: public brk_ev
{
public:
  brk_rs(uint Iaddr,
	 enum brk_perm Iperm,
	 int  Ihit);

  virtual bool match(struct event_rec *ev);
};


/* 
 * READ CODE
 */

class brk_rc: public brk_ev
{
public:
  brk_rc(uint Iaddr,
	 enum brk_perm Iperm,
	 int  Ihit);

  virtual bool match(struct event_rec *ev);
};


/*
 * Collection of breakpoint sorted by address
 */

class brk_coll: public TSortedCollection
{
public:
  brk_coll(TIndex ALimit, TIndex ADelta);
  virtual void *KeyOf(void *Item);
  virtual int  Compare(void *Key1, void *Key2);

  virtual bool there_is_event(enum brk_event ev);
};


#endif

/* End of brkcl.h */
