/*
 * Simulator of MCS51 (cmd_brk.cc)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "i_string.h"

#include "uccl.h"
#include "globals.h"


/*
 * BREAKPOINT SET f|d addr [hit]
 *____________________________________________________________________________
 *
 */

bool
cmd_brk_setf(char *cmd, class t_uc *uc)
{
  char *s;
  uint addr;
  int  hit= 1;
  enum brk_perm perm;
  class brkcl *b;

  if ((s= strtok(NULL, delimiters)) == NULL)
    {
      fprintf(uc->cmd_out, "Permanency has not given\n");
      return(FALSE);
    }
  switch (*s)
    {
    case 'f': case 'F':
      perm= brkFIX;
      break;
    case 'd': case 'D':
      perm= brkDYNAMIC;
      break;
    default:
      fprintf(uc->cmd_out, "Unknow permanency\n");
      return(FALSE);
    }
  if ((s= strtok(NULL, delimiters)) == NULL)
    {
      fprintf(uc->cmd_out, "Address has not given\n");
      return(FALSE);
    }
  addr= (uint)strtol(s, NULL, 0);
  if ((s= strtok(NULL, delimiters)) != NULL)
    hit= strtol(s, NULL, 0);
  if (uc->fbrk_at(addr))
    fprintf(uc->cmd_out, "Breakpoint at %06x is already set.\n", addr);
  else
    {
      b= new brk_fetch(addr, perm, hit);
      uc->fbrk->Insert(b);
    }
  return(FALSE);
}


/*
 * BREAKPOINT SET EVENT wi|ri|wx|rx|ws|rs|rc f|d addr [hit]
 *____________________________________________________________________________
 *
 */

bool
cmd_brk_sete(char *cmd, class t_uc *uc)
{
  char *s;
  char *id;
  uint addr;
  int  hit= 1;
  enum brk_perm perm;
  class brk_ev *b= NULL;

  if ((id= strtok(NULL, delimiters)) == NULL)
    {
      fprintf(uc->cmd_out, "Event has not given.\n");
      return(FALSE);
    }
  if ((s= strtok(NULL, delimiters)) == NULL)
    {
      fprintf(uc->cmd_out, "Permanency has not given\n");
      return(FALSE);
    }
  switch (*s)
    {
    case 'f': case 'F':
      perm= brkFIX;
      break;
    case 'd': case 'D':
      perm= brkDYNAMIC;
      break;
    default:
      fprintf(uc->cmd_out, "Unknow permanency\n");
      return(FALSE);
    }
  if ((s= strtok(NULL, delimiters)) == NULL)
    {
      fprintf(uc->cmd_out, "Address has not given\n");
      return(FALSE);
    }
  addr= (uint)strtol(s, NULL, 0);
  if ((s= strtok(NULL, delimiters)) != NULL)
    hit= strtol(s, NULL, 0);
  else
    {
      if (!strcmp(id, "wi"))
	b= new brk_wi(addr, perm, hit);
      else if (!strcmp(id, "ri"))
	b= new brk_ri(addr, perm, hit);
      else if (!strcmp(id, "wx"))
	b= new brk_wx(addr, perm, hit);
      else if (!strcmp(id, "rx"))
	b= new brk_rx(addr, perm, hit);
      else if (!strcmp(id, "ws"))
	b= new brk_ws(addr, perm, hit);
      else if (!strcmp(id, "rs"))
	b= new brk_rs(addr, perm, hit);
      else if (!strcmp(id, "rc"))
	b= new brk_rc(addr, perm, hit);
      else
	fprintf(uc->cmd_out, "Event %s is unknown.\n", id);
      if (b)
	uc->ebrk->Insert(b);
    }
  return(FALSE);
}


/*
 * BREAKPOINT DELETE addr
 *____________________________________________________________________________
 *
 */

bool
cmd_brk_delf(char *cmd, class t_uc *uc)
{
  uint addr;
  char *s;

  if ((s= strtok(NULL, delimiters)) == NULL)
    fprintf(uc->cmd_out, "Address has not given.\n");
  else
    {
      addr= (uint)strtol(s, NULL, 0);
      if (uc->fbrk_at(addr) == NULL)
	fprintf(uc->cmd_out, "No breakpoint at %06x\n", addr);
      else
	uc->rm_fbrk(addr);
    }
  return(FALSE);
}


/*
 * BREAKPOINT DELETE EVENT wi|ri|wx|rx|ws|rs|rc addr
 *____________________________________________________________________________
 *
 */

bool
cmd_brk_dele(char *cmd, class t_uc *uc)
{
  uint addr;
  char *s, *id;

  if ((id= strtok(NULL, delimiters)) == NULL)
    {
      fprintf(uc->cmd_out, "Event has not given.\n");
      return(FALSE);
    }
  if ((s= strtok(NULL, delimiters)) == NULL)
    fprintf(uc->cmd_out, "Address has not given.\n");
  else
    {
      addr= (uint)strtol(s, NULL, 0);
      if (uc->ebrk_at(addr, id) == NULL)
	fprintf(uc->cmd_out, "No %s breakpoint at %06x\n", id, addr);
      else
	uc->rm_ebrk(addr, id);
    }
  return(FALSE);
}


/*
 * BREAKPOINT DELETE ALL
 *____________________________________________________________________________
 *
 */

bool
cmd_brk_delall(char *cmd, class t_uc *uc)
{
  while (uc->fbrk->GetCount())
    uc->fbrk->AtFree(0);
  while (uc->ebrk->GetCount())
    uc->ebrk->AtFree(0);
  return(FALSE);
}


/*
 * BREAKPOINT LIST
 *____________________________________________________________________________
 *
 */

bool
cmd_brk_lst(char *cmd, class t_uc *uc)
{
  class brk_fetch *fb;
  class brk_ev *eb;
  int i;
  char *s;

  for (i= 0; i < uc->fbrk->GetCount(); i++)
    {
      fb= (class brk_fetch *)(uc->fbrk->At(i));
      s = uc->disass(fb->addr, NULL);
      fprintf(uc->cmd_out, "%c %d(%d) %06x %02x %s\n",
	      (fb->perm==brkFIX)?'F':'D',
	      fb->hit, fb->cnt,
	      fb->addr, uc->EROM[fb->addr], s);
      free(s);
    }
  for (i= 0; i < uc->ebrk->GetCount(); i++)
    {
      eb= (class brk_ev *)(uc->ebrk->At(i));
      fprintf(uc->cmd_out, "%c %d(%d) %06x %s\n",
	      (eb->perm==brkFIX)?'F':'D',
	      eb->hit, eb->cnt,
	      eb->addr, eb->id);
    }
  return(FALSE);
}


/* End of cmd_brk.cc */
