/*
 * Simulator of MCS51 (itsrc.cc)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include "i_string.h"

#include "itsrccl.h"
#include "pobj.h"
#include "stypes.h"


/*
 * Making a new object to represent source of an interrupt
 *____________________________________________________________________________
 *
 */

it_src::it_src(uchar Iie_mask,
	       uchar Isrc_reg,
	       uchar Isrc_mask,
	       uint  Iaddr,
	       bool  Iclr_bit,
	       char  *Iname):
  TObject()
{
  ie_mask = Iie_mask;
  src_reg = Isrc_reg;
  src_mask= Isrc_mask;
  addr    = Iaddr;
  clr_bit = Iclr_bit;
  if (Iname != NULL)
    name= strdup(Iname);
  else
    name= strdup("unknown");
  active= TRUE;
}


/*
 * Disposing dynamic fields
 *____________________________________________________________________________
 *
 */

it_src::~it_src(void)
{
  free(name);
}


/*
 * Activate/deactivate the interrupt
 *____________________________________________________________________________
 *
 */

bool
it_src::is_active(void)
{
  return(active);
}


void
it_src::set_active_status(bool Aactive)
{
  active= Aactive;
}

void
it_src::activate(void)
{
  set_active_status(TRUE);
}

void
it_src::deactivate(void)
{
  set_active_status(FALSE);
}


/* End of itsrc.cc */
