/*
 * Simulator of MCS51 (where.cc)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "i_string.h"

#include "stypes.h"
#include "uccl.h"
#include "globals.h"
#include "cmdutil.h"
#include "dump.h"


/*
 * Searching for a string in memory
 *____________________________________________________________________________
 *
 */

static void
where_memory(uchar *what, int size, class t_uc *uc, bool cs)
{
  char *s;
  uchar *str;
  int  len, i, found;
  int  start;

  if ((s= strtok(NULL, "\0")) == NULL)
    return;
  str= (uchar *)proc_escape(s, &len);
  // assume len < size!
  if (!cs)
    for (i= 0; i < len; i++)
      str[i]= toupper(str[i]);
  start= 0;
  while (start < size-len)
    {
      uint tmp= start;
      found= TRUE;
      for (i= 0; found && (i<len); i++)
	found= str[i] == (cs?what[start+i]:toupper(what[start+i]));
      if (found)
	{
	  dump_memory(what, size, &tmp, start+len-1, 8, uc->cmd_out);
	  start+= len;
	}
      else
	start++;
    }
  free(str);
}


/*
 * WHERE IRAM string
 *____________________________________________________________________________
 *
 */

bool
cmd_where_iram(char *cmd, class t_uc *uc)
{
  where_memory(uc->IRAM, IRAM_SIZE, uc, FALSE);
  return(FALSE);
}

bool
cmd_Where_iram(char *cmd, class t_uc *uc)
{
  where_memory(uc->IRAM, IRAM_SIZE, uc, TRUE);
  return(FALSE);
}


/*
 * WHERE XRAM string
 *____________________________________________________________________________
 *
 */

bool
cmd_where_xram(char *cmd, class t_uc *uc)
{
  uc->eram2xram();
  where_memory(uc->XRAM, XRAM_SIZE, uc, FALSE);
  return(FALSE);
}

bool
cmd_Where_xram(char *cmd, class t_uc *uc)
{
  uc->eram2xram();
  where_memory(uc->XRAM, XRAM_SIZE, uc, TRUE);
  return(FALSE);
}


/*
 * WHERE CODE string
 *____________________________________________________________________________
 *
 */

bool
cmd_where_code(char *cmd, class t_uc *uc)
{
  where_memory(uc->EROM, EROM_SIZE, uc, FALSE);
  return(FALSE);
}

bool
cmd_Where_code(char *cmd, class t_uc *uc)
{
  where_memory(uc->EROM, EROM_SIZE, uc, TRUE);
  return(FALSE);
}


/* End of where.cc */
